#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include"SimpleImage.h"
#include"ppmio.h"
#include"pgmio.h"
#include"ColorImage.h"
#include"otsu.h"
#include"PoissonIE.h"

Image* getMask(char *filename){
  Image *tmp = new Image();
  getPGM(tmp,filename);
  /* preprocessing */
  
   int HISTSIZE=256;
  Otsu *myotsu = new Otsu(HISTSIZE);

  double threshold = ((double)(myotsu->find_otsu_threshold(tmp)));
  int i,j;
  for(i=0;i<tmp->sy;i++)
    for(j=0;j<tmp->sx;j++){
      if(tmp->img[i][j]>=threshold){
	tmp->img[i][j] = 255.0;
      }else{
	tmp->img[i][j] = 0.0;
      }
    }
      delete myotsu;

  /* end preprocessing */
  return tmp;
}



void setMinMax(Image *in,int *min,int *max){
  int i,j;
  bool firstcheck=true;
  for(i=0;i<in->sy;i++)
    for(j=0;j<in->sx;j++){
      if(in->img[i][j]==255.0){
	if(firstcheck){
	  max[0] = min[0] = j;
	  max[1] = min[1] = i;
	  firstcheck=false;
	}else{
	  if(min[0]>j)min[0]=j;
	  if(min[1]>i)min[1]=i;
	  if(max[0]<j)max[0]=j;
	  if(max[1]<i)max[1]=i;
	  
	  
	
	}
      }
      
    }
  min[0]--;if(min[0]<0)min[0]=0;
  min[1]--;if(min[1]<0)min[1]=0;
  max[0]++;if(max[0]>in->sx-1)max[0]=in->sx-1;
  max[1]++;if(max[1]>in->sy-1)max[1]=in->sy-1;
  

}


int main(int argc,char *argv[]){
  
  ColorImage* Source = new ColorImage(argv[1]);
  Image *Mask = getMask(argv[2]);
  
  
  
  
  ColorImage* Target = new ColorImage(argv[3]);

  if(!(Source->sx==Mask->sx&&Mask->sx==Target->sx&&
      Source->sy==Mask->sy&&Mask->sy==Target->sy)){
  
    printf("All image size should be same !\n");
    delete Source;
    delete Target;
    delete Mask;
    return 0;
  
  }
  
  
  
  
  int i,j;
  ColorImage *out = new ColorImage(Target->sx,Target->sy);
  for(i=0;i<Target->sy;i++)
    for(j=0;j<Target->sx;j++){
      out->R->img[i][j] = Target->R->img[i][j];
      out->G->img[i][j] = Target->G->img[i][j];
      out->B->img[i][j] = Target->B->img[i][j];
    }
  int *min = new int[2];
  int *max = new int[2];
  setMinMax(Mask,min,max);
  
  /* main processing */
  int ssx = max[0]-min[0]+1;
  int ssy = max[1]-min[1]+1;
  
  ColorImage* Source2 = new ColorImage(ssx,ssy);
  
  Image *Mask2 = new Image(ssx,ssy);
  ColorImage* Target2 = new ColorImage(ssx,ssy);
  
  ColorImage *out2 = new ColorImage(ssx,ssy);
  
  for(i=0;i<ssy;i++){
    int i2 = i+min[1];
    for(j=0;j<ssx;j++){
      int j2 = j+min[0];
      Source2->R->img[i][j] = Source->R->img[i2][j2];
      Source2->G->img[i][j] = Source->G->img[i2][j2];
      Source2->B->img[i][j] = Source->B->img[i2][j2];
      
      Target2->R->img[i][j] = Target->R->img[i2][j2];
      Target2->G->img[i][j] = Target->G->img[i2][j2];
      Target2->B->img[i][j] = Target->B->img[i2][j2];
      
      Mask2->img[i][j] = Mask->img[i2][j2];
    }
  }
  double Alpha = atof(argv[5]);
  double Beta = atof(argv[6]);
  
  
  PoissonIE *pie = new PoissonIE(Source2->sx,Source2->sy);
  
  pie->solve(Source2->R,Target2->R,Mask2,out2->R,Alpha,Beta);
  pie->solve(Source2->G,Target2->G,Mask2,out2->G,Alpha,Beta);
  pie->solve(Source2->B,Target2->B,Mask2,out2->B,Alpha,Beta);
  
 
  for(i=0;i<ssy;i++){
    int i2 = i+min[1];
    for(j=0;j<ssx;j++){
      int j2 = j+min[0];
      out->R->img[i2][j2] = out2->R->img[i][j];
      out->G->img[i2][j2] = out2->G->img[i][j];
      out->B->img[i2][j2] = out2->B->img[i][j];
      
    }
  }
  
 



  out->saveColorImagePPM(argv[4]);
  
  
  delete pie;
  
  delete [] min;
  delete [] max;
  
  delete Source;
  delete Target;
  delete Mask;
  delete out;

  delete Source2;
  delete Target2;
  delete Mask2;
  delete out2;

  return 0;
}


